==> 0.10.1.tar.gz: test/mirrors/field_mirror_test.dart <==
@TestGroup('FieldMirror')
library fields.test.mirrors.field_mirror;

import 'dart:async';
import 'dart:mirrors';
import 'package:bench/bench.dart';
import 'package:unittest/unittest.dart' hide Description;
import 'package:fields/mirrors.dart';
import 'package:fields/model.dart';

main() => reflectTests();

// -----------------------------------------------------------------------------
// Setup
// -----------------------------------------------------------------------------

HasFields hasFields;
HasMetadata hasMetadata;
InstanceMirror hasFieldsMirror;
InstanceMirror hasMetadataMirror;
@Setup
void setup() {
  FieldSet.factory = reflectFields;
  hasFields = new HasFields();
  hasMetadata = new HasMetadata();
  hasFieldsMirror = reflect(hasFields);
  hasMetadataMirror = reflect(hasMetadata);
}

@Teardown
void teardown() {
  FieldSet.factory = FieldSet.defaultFactory;
  atomicTypes.remove(AtomicType);
}

class CustomType {
  int i;
  int get ci => i;
      set ci(int v) => i = v;
  String s;  
  String toString() => 'custom($i,$ci,$s)';
}

class HasFields {
  int i = 7;
  int get ri => 42;
  num n = 3.14;
  final num rn = 6.28;
  double d = 1024.4201;
  final double rd = 1.024;
  String s = 'snarf';
  final String rs = 'immutable';
  List li = [ 1, 2, 3, 4];    
  Map m = { 'a' : 1, 'b' : 2 };
  CustomType c = new CustomType();
}

class AtomicType {
  int i = 7;
  // Allocating a new instance from this getter will lead to a stack overflow
  // during field reflection unless this type is registered as an atomic type.
  AtomicType get overflow => new AtomicType();
}

class HasAtomicTypes {
  AtomicType atom = new AtomicType();
}

class HasMetadata {
  @Ignore
  int i = 14;
  @DisplayName('PI')
  num n = 3.14;  
  @ReadOnly
  int ri = 42;
  @Description('A mutable flag.')
  bool b = false;
}

// -----------------------------------------------------------------------------
// Basic
// -----------------------------------------------------------------------------

@Test('Test the construction of an int field.')
void testInt() {
  final f = new FieldMirror.forVariable(#i, hasFieldsMirror);    
  expect(f.value, equals(7));
  expect(f.declaredName, equals(#i));
  expect(f.type, equals(int));
  expect(f.isReadOnly, isFalse);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a read-only int field.')
void testIntReadOnly() {
  final f = new FieldMirror.forGetter(#ri, hasFieldsMirror);
  expect(f.value, equals(42));    
  expect(f.declaredName, equals(#ri));
  expect(f.type, equals(int));
  expect(f.isReadOnly, isTrue);
  expect(f.isGetter, isTrue);
  expect(f.isVariable, isFalse);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a num field.')
void testNum() {
  final f = new FieldMirror.forVariable(#n, hasFieldsMirror);    
  expect(f.value, equals(3.14));  
  expect(f.declaredName, equals(#n));
  expect(f.type, equals(num));
  expect(f.isReadOnly, isFalse);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a read-only num field.')
void testNumReadOnly() {
  final f = new FieldMirror.forVariable(#rn, hasFieldsMirror);    
  expect(f.value, equals(6.28));  
  expect(f.declaredName, equals(#rn));
  expect(f.type, equals(num));
  expect(f.isReadOnly, isTrue);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a double field.')
void testDouble() {
  final f = new FieldMirror.forVariable(#d, hasFieldsMirror);    
  expect(f.value, equals(1024.4201));  
  expect(f.declaredName, equals(#d));
  expect(f.type, equals(double));
  expect(f.isReadOnly, isFalse);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a read-only double field.')
void testDoubleReadOnly() {
  final f = new FieldMirror.forVariable(#rd, hasFieldsMirror);    
  expect(f.value, equals(1.024));  
  expect(f.declaredName, equals(#rd));
  expect(f.type, equals(double));
  expect(f.isReadOnly, isTrue);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a string field.')
void testString() {
  final f = new FieldMirror.forVariable(#s, hasFieldsMirror);    
  expect(f.value, equals('snarf'));  
  expect(f.declaredName, equals(#s));
  expect(f.type, equals(String));
  expect(f.isReadOnly, isFalse);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a read-only string field.')
void testStringReadOnly() {
  final f = new FieldMirror.forVariable(#rs, hasFieldsMirror);
  expect(f.value, equals('immutable'));  
  expect(f.declaredName, equals(#rs));
  expect(f.type, equals(String));
  expect(f.isReadOnly, isTrue);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a list field.')
void testList() {
  final f = new FieldMirror.forVariable(#li, hasFieldsMirror);    
  expect(f.value, equals([1, 2, 3, 4]));  
  expect(f.declaredName, equals(#li));
  expect(f.type, equals(List));
  expect(f.isReadOnly, isFalse);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children.length, equals(4));
  expect(f.children.elementAt(0).name, equals('[0]'));
  expect(f.children.elementAt(0).value, equals(1));
  expect(f.children.elementAt(1).name, equals('[1]'));
  expect(f.children.elementAt(1).value, equals(2));
  expect(f.children.elementAt(2).name, equals('[2]'));
  expect(f.children.elementAt(2).value, equals(3));
  expect(f.children.elementAt(3).name, equals('[3]'));
  expect(f.children.elementAt(3).value, equals(4));
}

@Test('Test the construction of a map field.')
void testMap() {
  final f = new FieldMirror.forVariable(#m, hasFieldsMirror);    
  expect(f.value, equals({ 'a' : 1, 'b' : 2 }));  
  expect(f.declaredName, equals(#m));
  expect(f.type, equals(Map));
  expect(f.isReadOnly, isFalse);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test that setting a value on a read-only field throws.')
@ExpectError(isAssertionError)
void testSetValueReadOnlyThrows() {
  final f = new FieldMirror.forGetter(#ri, hasFieldsMirror);
  f.value = 13;
}

@Test('Test that setting a value adds a new data event to the stream.')
void testSetValue() {
  final f = new FieldMirror.forVariable(#i, hasFieldsMirror);
  f.onValueChange.listen(expectAsync((v) {
    expect(v, equals(8));
  }));  
  f.value = 8;
  expect(f.value, equals(8));
}

@Test('Test that no events are added to the values stream when setting a new'
      ' value if there are no listeners.')
void testSetValueNoValuesListener() {
  final f = new FieldMirror.forVariable(#i, hasFieldsMirror);  
  f.value = 8;  
  // Delay to allow the async refresh of children to complete
  new Future.value(f.value).then(expectAsync((v) {
    expect(v, equals(8));
    final noop = expectAsync(() {});
    
    // Now attach a listener.
    f.onValueChange.listen((_) => fail('No values should be buffered.'));
    
    // Delay test completion to make sure no events are flushed to listener.
    scheduleMicrotask(noop);
  }));
}

// -----------------------------------------------------------------------------
// Metadata
// -----------------------------------------------------------------------------

@Test('Test that factory constructors return null if @Ignore is present.')
void testIgnore() {
  final f = new FieldMirror.forVariable(#i, hasMetadataMirror);
  expect(f, isNull);  
}

@Test('Test that the @Name annotation is respected.')
void testName() {
  final f = new FieldMirror.forVariable(#n, hasMetadataMirror);
  expect(f.declaredName, equals(#n));
  expect(f.name, equals('n'));
  expect(f.displayName, equals('PI'));
  expect(f.annotations, isEmpty);
}

@Test('Test that the @Description annotation is respected.')
void testDescription() {
  final f = new FieldMirror.forVariable(#b, hasMetadataMirror);
  expect(f.description, equals('A mutable flag.'));
  expect(f.annotations, isEmpty);
}

@Test('Test that the @ReadOnly annotation is respected.')
void testReadOnly() {
  final f = new FieldMirror.forVariable(#ri, hasMetadataMirror);
  expect(f.declaredName, equals(#ri));
  expect(f.isReadOnly, isTrue);
}

// -----------------------------------------------------------------------------
// Custom Type
// -----------------------------------------------------------------------------

@Test('Test the construction of a custom type field.')
void testCustomType() {
  final f = new FieldMirror.forVariable(#c, hasFieldsMirror);    
  expect(f.value, const isInstanceOf<CustomType>());  
  expect(f.declaredName, equals(#c));
  expect(f.type, equals(CustomType));
  expect(f.isReadOnly, isFalse);
  expect(f.children.length, equals(3));
}

@Test('Test that a child set value affects parent + sibling values.')
void testSetChildValue() {
  final f = new FieldMirror.forVariable(#c, hasFieldsMirror);
  
  // Set value listeners on the children
  final i = f.children.firstWhere((c) => c.declaredName == #i);
  i.onValueChange.listen(expectAsync((v) => expect(v, equals(9))));
  
  final ci = f.children.firstWhere((c) => c.declaredName == #ci);
  ci.onValueChange.listen(expectAsync((v) => expect(v, equals(9))));
  
  final s = f.children.firstWhere((c) => c.declaredName == #s);
  s.onValueChange.listen((v) => 
      fail('s did not change, expect no stream data.'));

  // Set value listener on the parent
  f.onValueChange.listen(expectAsync((v) => 
      expect('$v', equals('custom(9,9,null)'))));
  
  // Set the value of 'i'
  i.value = 9;
}

// -----------------------------------------------------------------------------
// Atomic Type
// -----------------------------------------------------------------------------

@Test('Test that reflecting an unregistered atomic type causes stack overflow.')
@ExpectError(isStackOverflowError)
void testAtomicTypeStackOverflow() {
  new FieldMirror.forVariable(#atom, reflect(new HasAtomicTypes()));
}

@Test('Test that an atomic type may be reflected when registered.')
void testAddAtomicType() {
  atomicTypes.add(AtomicType);
  final o = new HasAtomicTypes();
  final atom = new FieldMirror.forVariable(#atom, reflect(o));
  expect(atom.type, equals(AtomicType));
  expect(atom.children, isEmpty);
}

==> 0.11.0.tar.gz: test/mirrors/field_mirror_test.dart <==
@TestGroup('FieldMirror')
library fields.test.mirrors.field_mirror;

import 'dart:async';
import 'dart:mirrors';
import 'package:bench/bench.dart';
import 'package:unittest/unittest.dart' hide Description;
import 'package:fields/mirrors.dart';
import 'package:fields/model.dart';

main() => reflectTests();

// -----------------------------------------------------------------------------
// Setup
// -----------------------------------------------------------------------------

HasFields hasFields;
HasMetadata hasMetadata;
InstanceMirror hasFieldsMirror;
InstanceMirror hasMetadataMirror;
@Setup
void setup() {
  FieldSet.factory = reflectFields;
  hasFields = new HasFields();
  hasMetadata = new HasMetadata();
  hasFieldsMirror = reflect(hasFields);
  hasMetadataMirror = reflect(hasMetadata);
}

@Teardown
void teardown() {
  FieldSet.factory = FieldSet.defaultFactory;
  atomicTypes.remove(AtomicType);
}

class CustomType {
  int i;
  int get ci => i;
      set ci(int v) => i = v;
  String s;  
  String toString() => 'custom($i,$ci,$s)';
  Nested n = new Nested();
}

class Nested {
  String s;
  String toString() => 'nested($s)';
}

class HasFields {
  int i = 7;
  int get ri => 42;
  num n = 3.14;
  final num rn = 6.28;
  double d = 1024.4201;
  final double rd = 1.024;
  String s = 'snarf';
  final String rs = 'immutable';
  List li = [ 1, 2, 3, 4];    
  Map m = { 'a' : 1, 'b' : 2 };
  CustomType c = new CustomType();
}

class AtomicType {
  int i = 7;
  // Allocating a new instance from this getter will lead to a stack overflow
  // during field reflection unless this type is registered as an atomic type.
  AtomicType get overflow => new AtomicType();
}

class HasAtomicTypes {
  AtomicType atom = new AtomicType();
}

class HasMetadata {
  @Ignore
  int i = 14;
  @DisplayName('PI')
  num n = 3.14;  
  @ReadOnly
  int ri = 42;
  @Description('A mutable flag.')
  bool b = false;
}

// -----------------------------------------------------------------------------
// Basic
// -----------------------------------------------------------------------------

@Test('Test the construction of an int field.')
void testInt() {
  final f = new FieldMirror.forVariable(#i, hasFieldsMirror);    
  expect(f.value, equals(7));
  expect(f.declaredName, equals(#i));
  expect(f.type, equals(int));
  expect(f.isReadOnly, isFalse);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a read-only int field.')
void testIntReadOnly() {
  final f = new FieldMirror.forGetter(#ri, hasFieldsMirror);
  expect(f.value, equals(42));    
  expect(f.declaredName, equals(#ri));
  expect(f.type, equals(int));
  expect(f.isReadOnly, isTrue);
  expect(f.isGetter, isTrue);
  expect(f.isVariable, isFalse);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a num field.')
void testNum() {
  final f = new FieldMirror.forVariable(#n, hasFieldsMirror);    
  expect(f.value, equals(3.14));  
  expect(f.declaredName, equals(#n));
  expect(f.type, equals(num));
  expect(f.isReadOnly, isFalse);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a read-only num field.')
void testNumReadOnly() {
  final f = new FieldMirror.forVariable(#rn, hasFieldsMirror);    
  expect(f.value, equals(6.28));  
  expect(f.declaredName, equals(#rn));
  expect(f.type, equals(num));
  expect(f.isReadOnly, isTrue);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a double field.')
void testDouble() {
  final f = new FieldMirror.forVariable(#d, hasFieldsMirror);    
  expect(f.value, equals(1024.4201));  
  expect(f.declaredName, equals(#d));
  expect(f.type, equals(double));
  expect(f.isReadOnly, isFalse);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a read-only double field.')
void testDoubleReadOnly() {
  final f = new FieldMirror.forVariable(#rd, hasFieldsMirror);    
  expect(f.value, equals(1.024));  
  expect(f.declaredName, equals(#rd));
  expect(f.type, equals(double));
  expect(f.isReadOnly, isTrue);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a string field.')
void testString() {
  final f = new FieldMirror.forVariable(#s, hasFieldsMirror);    
  expect(f.value, equals('snarf'));  
  expect(f.declaredName, equals(#s));
  expect(f.type, equals(String));
  expect(f.isReadOnly, isFalse);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a read-only string field.')
void testStringReadOnly() {
  final f = new FieldMirror.forVariable(#rs, hasFieldsMirror);
  expect(f.value, equals('immutable'));  
  expect(f.declaredName, equals(#rs));
  expect(f.type, equals(String));
  expect(f.isReadOnly, isTrue);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a list field.')
void testList() {
  final f = new FieldMirror.forVariable(#li, hasFieldsMirror);    
  expect(f.value, equals([1, 2, 3, 4]));  
  expect(f.declaredName, equals(#li));
  expect(f.type, equals(List));
  expect(f.isReadOnly, isFalse);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children.length, equals(4));
  expect(f.children.elementAt(0).name, equals('[0]'));
  expect(f.children.elementAt(0).value, equals(1));
  expect(f.children.elementAt(1).name, equals('[1]'));
  expect(f.children.elementAt(1).value, equals(2));
  expect(f.children.elementAt(2).name, equals('[2]'));
  expect(f.children.elementAt(2).value, equals(3));
  expect(f.children.elementAt(3).name, equals('[3]'));
  expect(f.children.elementAt(3).value, equals(4));
}

@Test('Test the construction of a map field.')
void testMap() {
  final f = new FieldMirror.forVariable(#m, hasFieldsMirror);    
  expect(f.value, equals({ 'a' : 1, 'b' : 2 }));  
  expect(f.declaredName, equals(#m));
  expect(f.type, equals(Map));
  expect(f.isReadOnly, isFalse);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test that setting a value on a read-only field throws.')
@ExpectError(isAssertionError)
void testSetValueReadOnlyThrows() {
  final f = new FieldMirror.forGetter(#ri, hasFieldsMirror);
  f.value = 13;
}

@Test('Test that setting a value adds a new data event to the stream.')
void testSetValue() {
  final f = new FieldMirror.forVariable(#i, hasFieldsMirror);
  f.onValueChange.listen(expectAsync((v) {
    expect(v, equals(8));
  }));  
  f.value = 8;
  expect(f.value, equals(8));
}

@Test('Test that no events are added to the values stream when setting a new'
      ' value if there are no listeners.')
void testSetValueNoValuesListener() {
  final f = new FieldMirror.forVariable(#i, hasFieldsMirror);  
  f.value = 8;  
  // Delay to allow the async refresh of children to complete
  new Future.value(f.value).then(expectAsync((v) {
    expect(v, equals(8));
    final noop = expectAsync(() {});    
    // Now attach a listener.
    f.onValueChange.listen((_) => fail('No values should be buffered.'));    
    // Delay test completion to make sure no events are flushed to listener.
    scheduleMicrotask(noop);
  }));
}

// -----------------------------------------------------------------------------
// Metadata
// -----------------------------------------------------------------------------

@Test('Test that factory constructors return null if @Ignore is present.')
void testIgnore() {
  final f = new FieldMirror.forVariable(#i, hasMetadataMirror);
  expect(f, isNull);  
}

@Test('Test that the @Name annotation is respected.')
void testName() {
  final f = new FieldMirror.forVariable(#n, hasMetadataMirror);
  expect(f.declaredName, equals(#n));
  expect(f.name, equals('n'));
  expect(f.displayName, equals('PI'));
  expect(f.annotations, isEmpty);
}

@Test('Test that the @Description annotation is respected.')
void testDescription() {
  final f = new FieldMirror.forVariable(#b, hasMetadataMirror);
  expect(f.description, equals('A mutable flag.'));
  expect(f.annotations, isEmpty);
}

@Test('Test that the @ReadOnly annotation is respected.')
void testReadOnly() {
  final f = new FieldMirror.forVariable(#ri, hasMetadataMirror);
  expect(f.declaredName, equals(#ri));
  expect(f.isReadOnly, isTrue);
}

// -----------------------------------------------------------------------------
// Custom Type
// -----------------------------------------------------------------------------

@Test('Test the construction of a custom type field.')
void testCustomType() {
  final f = new FieldMirror.forVariable(#c, hasFieldsMirror);    
  expect(f.value, const isInstanceOf<CustomType>());  
  expect(f.declaredName, equals(#c));
  expect(f.type, equals(CustomType));
  expect(f.isReadOnly, isFalse);
  expect(f.children.length, equals(4));
}

@Test('Test that a child set value affects parent + sibling values.')
void testSetChildValue() {
  // Set value listener on the parent
  final p = new FieldMirror.forVariable(#c, hasFieldsMirror);
  p.onValueChange.listen(expectAsync((v) => 
      expect('$v', equals('custom(9,9,null)'))));
  // Set value listeners on the children
  final i = p.children.firstWhere((c) => c.declaredName == #i);
  i.onValueChange.listen(expectAsync((v) => expect(v, equals(9))));  
  final ci = p.children.firstWhere((c) => c.declaredName == #ci);
  ci.onValueChange.listen(expectAsync((v) => expect(v, equals(9))));  
  final s = p.children.firstWhere((c) => c.declaredName == #s);
  s.onValueChange.listen((v) => 
      fail('s did not change, expect no stream data.'));  
  // Set the value of 'i'
  i.value = 9;
}

@Test('Test that a grandchild set value affects parent + grandparent values.')
void testSetGrandchildValue() {
  // Grandparent
  final gp = new FieldMirror.forVariable(#c, hasFieldsMirror);
  gp.onValueChange.listen(expectAsync((v) => 
      expect('$v', equals('custom(null,null,null)'))));  
  // Parent
  final p = gp.children.firstWhere((c) => c.declaredName == #n);
  p.onValueChange.listen(expectAsync((v) => 
      expect('$v', equals('nested(snarf)'))));  
  // Child
  final s = p.children.firstWhere((c) => c.declaredName == #s);
  s.onValueChange.listen(expectAsync((v) => expect(v, equals('snarf'))));  
  s.value = 'snarf';
}

// -----------------------------------------------------------------------------
// Atomic Type
// -----------------------------------------------------------------------------

@Test('Test that reflecting an unregistered atomic type causes stack overflow.')
@ExpectError(isStackOverflowError)
void testAtomicTypeStackOverflow() {
  new FieldMirror.forVariable(#atom, reflect(new HasAtomicTypes()));
}

@Test('Test that an atomic type may be reflected when registered.')
void testAddAtomicType() {
  atomicTypes.add(AtomicType);
  final o = new HasAtomicTypes();
  final atom = new FieldMirror.forVariable(#atom, reflect(o));
  expect(atom.type, equals(AtomicType));
  expect(atom.children, isEmpty);
}
